@@ -44,8 +44,9 @@ class AnnotationCCodeWriter(CCodeWriter):
         if pos is not None:
             CCodeWriter.mark_pos(self, pos)
         if self.last_pos:
-            code = self.code.get(self.last_pos[1], "")
-            self.code[self.last_pos[1]] = code + self.annotation_buffer.getvalue()
+            pos_code = self.code.setdefault(self.last_pos[0].get_description(),{})
+            code = pos_code.get(self.last_pos[1], "")
+            pos_code[self.last_pos[1]] = code + self.annotation_buffer.getvalue()
         self.annotation_buffer = StringIO()
         self.last_pos = pos
 
@@ -63,15 +64,16 @@ class AnnotationCCodeWriter(CCodeWriter):
             lines[k] = line
         f.close()
         all = []
-        for pos, item in self.annotations:
-            if pos[0] == source_filename:
-                start = item.start()
-                size, end = item.end()
-                if size:
-                    all.append((pos, start))
-                    all.append(((source_filename, pos[1], pos[2]+size), end))
-                else:
-                    all.append((pos, start+end))
+        if False:
+            for pos, item in self.annotations:
+                if pos[0].filename == source_filename:
+                    start = item.start()
+                    size, end = item.end()
+                    if size:
+                        all.append((pos, start))
+                        all.append(((source_filename, pos[1], pos[2]+size), end))
+                    else:
+                        all.append((pos, start+end))
                 
         all.sort()
         all.reverse()
@@ -125,7 +127,7 @@ function toggleDiv(id) {
         """)
         f.write(u'<body>\n')
         f.write(u'<p>Generated by Cython %s on %s\n' % (Version.version, time.asctime()))
-        c_file = Utils.encode_filename(os.path.basename(target_filename))
+        c_file = Utils.decode_filename(os.path.basename(target_filename))
         f.write(u'<p>Raw output: <a href="%s">%s</a>\n' % (c_file, c_file))
         k = 0
         
@@ -136,14 +138,17 @@ function toggleDiv(id) {
         error_goto = re.compile(ur'((; *if .*)? \{__pyx_filename = .*goto __pyx_L\w+;\})')
         refnanny = re.compile(u'(__Pyx_X?(GOT|GIVE)REF|__Pyx_RefNanny[A-Za-z]+)')
         
+        code_source_file = self.code[source_filename]
         for line in lines:
 
             k += 1
             try:
-                code = self.code[k]
+                code = code_source_file[k]
             except KeyError:
                 code = ''
-                
+            
+            code = code.replace('<', '<code><</code>')
+            
             code, py_c_api_calls = py_c_api.subn(ur"<span class='py_c_api'>\1</span>(", code)
             code, pyx_c_api_calls = pyx_c_api.subn(ur"<span class='pyx_c_api'>\1</span>(", code)
             code, py_macro_api_calls = py_marco_api.subn(ur"<span class='py_macro_api'>\1</span>(", code)
