@@ -36,6 +36,8 @@ Options:
   --line-directives              Produce #line directives pointing to the .pyx source
   --cplus                        Output a c++ rather than c file.
   --embed                        Embed the Python interpreter in a main() method.
+  -2                             Compile based on Python-2 syntax and code semantics.
+  -3                             Compile based on Python-3 syntax and code semantics.
   -X, --directive <name>=<value>[,<name=value,...] Overrides a compiler directive
 """
 
@@ -114,11 +116,17 @@ def parse_command_line(args):
                 Options.convert_range = True
             elif option == "--line-directives":
                 options.emit_linenums = True
+            elif option == '-2':
+                options.language_level = 2
+            elif option == '-3':
+                options.language_level = 3
             elif option in ("-X", "--directive"):
                 try:
-                    options.compiler_directives = Options.parse_directive_list(pop_arg(), relaxed_bool=True)
+                    options.compiler_directives = Options.parse_directive_list(
+                        pop_arg(), relaxed_bool=True,
+                        current_settings=options.compiler_directives)
                 except ValueError, e:
-                    sys.stderr.write("Error in compiler directive: %s\n" % e.message)
+                    sys.stderr.write("Error in compiler directive: %s\n" % e.args[0])
                     sys.exit(1)
             else:
                 sys.stderr.write("Unknown compiler flag: %s\n" % option)
